/*!
 * Copyright (c) 2015-2019 Cisco Systems, Inc. See LICENSE file.
 */

import {browserOnly, nodeOnly} from '@webex/test-helper-mocha';
import {assert} from '@webex/test-helper-chai';
import testUsers from '@webex/test-helper-test-users';
import WebexCore, {filterScope} from '@webex/webex-core';
import refreshCallback from '@webex/test-helper-refresh-callback';

/* eslint camelcase: [0] */

describe('webex-core', () => {
  describe('Credentials', () => {
    describe('Token', () => {
      let webex, user;

      before(() => testUsers.create({count: 1})
        .then(([u]) => {
          user = u;
        }));

      describe('#downscope()', () => {
        it('retrieves an access token with a subset of scopes', () => {
          webex = new WebexCore({credentials: user.token});
          const allScope = webex.credentials.config.scope;
          const apiScope = filterScope('spark:kms', allScope);

          return webex.credentials.supertoken.downscope('spark:kms')
            .then((downscopedToken) => downscopedToken.validate())
            .then((details) => assert.deepEqual(details.scope, ['spark:kms']))
            .then(() => webex.credentials.supertoken.downscope(apiScope))
            .then((downscopedToken) => downscopedToken.validate())
            .then((details) => assert.sameMembers(details.scope, apiScope.split(' ')))
            .then(() => assert.isRejected(webex.credentials.supertoken.downscope(allScope), /token: scope reduction requires a reduced scope/));
        });
      });

      describe('#refresh()', () => {
        nodeOnly(it)('refreshes the token, returning a new Token instance', () => {
          webex = new WebexCore({credentials: user.token});

          return webex.credentials.supertoken.refresh()
            .then((token2) => {
              assert.notEqual(token2.access_token, webex.credentials.supertoken.access_token);
              assert.equal(token2.refresh_token, webex.credentials.supertoken.refresh_token);
            });
        });

        browserOnly(it)('refreshes the token, returning a new Token instance', () => {
          webex = new WebexCore({
            credentials: user.token,
            config: {
              credentials: {
                refreshCallback
              }
            }
          });

          return webex.credentials.supertoken.refresh()
            .then((token2) => {
              assert.notEqual(token2.access_token, webex.credentials.supertoken.access_token);
              assert.equal(token2.refresh_token, webex.credentials.supertoken.refresh_token);
            });
        });
      });

      describe('#validate()', () => {
        it('shows the token\'s scopes', () => {
          webex = new WebexCore({credentials: user.token});

          return webex.credentials.supertoken.validate()
            .then((details) => {
              const detailScope = details.scope.sort();
              const localScope = webex.credentials.config.scope.split(' ').sort();

              assert.sameMembers(detailScope, localScope);
              assert.lengthOf(detailScope, localScope.length);
              assert.equal(details.clientId, webex.credentials.config.client_id);
            });
        });
      });

      // These tests have a bit of shared state, so revoke() needs to go last
      describe('#revoke()', () => {
        it('revokes the token', () => {
          webex = new WebexCore({credentials: user.token});

          return webex.credentials.supertoken.revoke()
            .then(() => {
              assert.isUndefined(webex.credentials.supertoken.access_token);
              assert.isDefined(webex.credentials.supertoken.refresh_token);
              assert.isUndefined(webex.credentials.supertoken.expires_in);
            });
        });
      });
    });
  });
});
